;Sprite Drawing Routine (SDR): 8 Bit Speed Version (White Transparency)
;by JL
;<JayEll64@aol.com>
;
;Inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;Outputs:
;	af=destroyed
;	bc=destroyed
;	de=destroyed
;	hl=pointer to byte after sprite
;	no other registers are changed
;If you want to save those registers that are destroyed, push/pop them!
;
;Sprite data should be stored as follows:
;.db (# rows/bytes/height of sprite)
;.db (bitmap of sprite)
;
;Size=58 bytes (+pushes/pops)
;The fastest (???) sprite routine found...  :)
;	(Hey!  Hard to beat 1500 clock cycles!)
;
;Notes:
;You can 'xor' the sprite to the video memory by changing both 'or (hl)'s
; with 'xor (hl)'s
;You can make this routine smaller by deleting the first 3 lines and
; accounting in your program the fact the the origin (0,0) is the *bottom*
; left corner (saves 4 bytes!).
;Also, if all your sprites are the same height, you can delete the next 2
; lines (ld e,(hl)\inc hl); change the 3rd line after that (ld c,e) to
; 'ld c,x', where x is the height of your sprites; and store all sprites
; with only the sprite data, leaving off that first byte that defines the
; height (saves 1 bytes + # of sprites!)
;
;Each instruction is commented with the number of clock cycles it takes.

SDR8SP:
 ld a,63		;7
 sub c			;4
 ld c,a			;4
 ld e,(hl)		;7
 inc hl			;6
 push hl		;11
 call FIND_PIXEL	;17
 ld b,e			;4
 ld de,VIDEO_MEM	;10
 add hl,de		;11

 add a,a		;4
 dec a			;4
 ld c,a			;4
 ld a,b			;4
 and %00000111		;4
 ld (&SDR8SP_PrepByte+1),a	;13+
;***114+ (+time for FIND_PIXEL) clock cycles up to this point***

SDR8SP_NewRow:
 ex (sp),hl		;4
 ld a,(hl)		;7
 inc hl			;6
 ex (sp),hl		;4

SDR8SP_PrepByte:
 jr 0			;12
 rlca\ rlca\ rlca\ rlca	;4 each=4 to 32 total (average=18)
 rlca\ rlca\ rlca\ rlca

 ld d,a			;4
 and c			;4
 or (hl)		;7
 ld (hl),a		;7
 inc hl			;6

 ld a,c			;4
 cpl			;4
 and d			;4
 or (hl)		;7
 ld (hl),a		;7

 ld de,$10-1		;10
 add hl,de		;11
;***126 average clock cycles from SDR8SP_NewRow to here***
 djnz SDR8SP_NewRow	;13 if jump, 8 if nop
 pop hl			;10
 ret			;10

;Average number of total clock cycles (for an 8x8 sprite)=
;	114+126*8+13*7+8+10+10+FIND_PIXEL=1241+FIND_PIXEL
;I estimate FIND_PIXEL should be about 200 clock cycles, so total number of
;	clock cycles for routine should be about *1500*.
;Number of sprites displayed in one second (theoretically)=
;	1500 clock cycles/sprite
;	-----------------------------	=	***4000 sprites/second***
;	6 000 000 clock cycles/second
;	(Remember, the Z80 is a 6MHz processor...)
;
;Of course, you couldn't possibly display that many sprites unless you lined
;	up 4000 SDR8SP's in a row...not likely :)  But it's still about 1500
;	clock cycles, so you should be able to display 2000 to 3000 sprites
;	a second pratically!